% Time advancement for panel flutter
% with an improved Runge-Kutta method


%% START ==========================================================

clc;
clear;
format long;
warning('on');


%% Parameters of calculation status

lambda = 550;   % non-dimensional dynamic pressure
Cpc = 1.22;     % cavity pressure coefficient

N = 10000;      % iteration step

%% Initial value

continue_tag = 0;
% 0: start from a specific initial value
% 1: continue from a specific state

u_initial=[...
    0,   0,  0,  0,  0,  0, ...
    0,   0,  0,  0,  0,  0  ...
    ];        % initial value when continue_tag = 0

%% Parameters of result

position = 0:0.01:1;        % points on panel, form generalized displacement to the displacement of the point
monitor  = [0.25, 0.75];    % position of displacement monitor, percentage

folder_name='RK___'; % result folder name

n_monitor=size(monitor,2);

%% Basic Parameters

M   = 2.0;           % Mach number
l   = 1.0;           % plate length
h   = 0.002;         % plate thickness

R0  = 0;              % initial in-plane force (non-dimensional)

beta = (M^2-1)^0.5;

%% Parameters of ROM

lambda_train = 600;  % non-dimensional dynamic pressure used in the training process
DT_train = 0.0005;   % time step used in the training process

n_mode = 6;

load('___resource_ARXmodel\database_myARXModel.mat');

na = myARXModel.na;
nb = myARXModel.nb;
nab = max(myARXModel.na,myARXModel.nb);

n_out = myARXModel.n_out;
n_in = myARXModel.n_in;

y_initial = myARXModel.initialValue;

THETA = myARXModel.THETA;

clear myARXModel;

%% 

fprintf('==============================================\n')
fprintf('%s\tStart\n',datetime(datetime,'Format','HH:mm:ss.SSS'));
fprintf('\t\t\t\t   = %g\n',lambda);
fprintf('\t\t\t\tCpc = %g\n',Cpc);
fprintf('----------------------------------------------\n')


switch continue_tag
    case 0
        % initial value mode
        
        fprintf('\t\t\t\tinitial value mode:\n');
        fprintf('\t\t\t\t%.1f , %.1f , %.1f\n',u_initial(1:3));
        fprintf('\t\t\t\t%.1f , %.1f , %.1f\n',u_initial(4:6));
        
        fprintf('\t\t\t\t----------------\n');
        
        fprintf('\t\t\t\t%.1f , %.1f , %.1f\n',u_initial(7:9));
        fprintf('\t\t\t\t%.1f , %.1f , %.1f\n',u_initial(10:12));
        
        % save initial value
        initialFileName='___Initial_value.txt';
        save(initialFileName,'u_initial','-ascii');
        
    case 1
        % continue mode
        
        fprintf('\t\t\t\tcontinue mode:\n');
        try
            continueFile = load('___resource_continueDATA/continueDATA.mat');
            continueDATA = continueFile.continueDATA;
            lambda_old = continueDATA.lambda_old;
        catch
            error('continueDATA reading excption.');
        end
        fprintf('\t\t\t\told lambda  %f\n',continueDATA.lambda_old);
        fprintf('\t\t\t\told pc      %.3f\n',continueDATA.Cpc_old);
        fprintf('\t\t\t\told DT      %.2e\n',continueDATA.DT_old);
        
        % save initial continueDATA file
        initialFileName=['___Initial_continue_',num2str(continueDATA.lambda_old),'.mat'];
        save(initialFileName,'continueDATA');

        if ~ismember('n_continue',fieldnames(continueDATA))
            continueDATA.n_continue = 1;
            warning('The compatibility check specifies the default continueDATA.n_continue');
        else
            % Note:nothing to do
        end
       
    otherwise
        error('unexpected input: restart_tag');
end

fprintf('----------------------------------------------\n')

%% 
        
% DT is structure time, DT_f is fluid time
% DT_f  =  DT    /  (M*(mu/(lambda_train*beta))^0.5);
% DT    =  DT_f  *  (M*(mu/(lambda      *beta))^0.5);
% timefactor = (M*(mu/(lambda_train*beta_plate))^0.5)/...
%              (M*(mu/(lambda      *beta_plate))^0.5);

timefactor = (lambda/lambda_train)^0.5;
DT = DT_train/timefactor;

%%

switch continue_tag
    case 0
        % Note:nothing to do
    case 1
        continueDATA.GD = ContinueCalcu_resample(continueDATA.DT_old,DT,continueDATA.GD);
        continueDATA.GF = ContinueCalcu_resample(continueDATA.DT_old,DT,continueDATA.GF);
    otherwise
        % Note:nothing to do
end

%% Add time axis

switch continue_tag
    case 0

        time = (0:DT:(N-1)*DT)';   % time

    case 1

        if (lambda == continueDATA.lambda_old)...
                &&(Cpc == continueDATA.Cpc_old)
            time = (DT:DT:N*DT)' + continueDATA.time;   % Time for continuous calculation of the same dynamic pressure
            continueDATA.n_continue = continueDATA.n_continue + 1; % Update the number of times to continued calculation
        else
            time = (0:DT:(N-1)*DT)';   % time
            continueDATA.n_continue = 1;
        end
        fprintf('\t\t\t\t=%g , %d th calculation.\n',lambda,continueDATA.n_continue);
        fprintf('----------------------------------------------\n')
        
    otherwise
        % Note:nothing to do
end


%% Prepare data

% generalized displacements and generalized forces required for the first iteration

simGD = zeros(N+nab+1,2*n_in);
simGF = zeros(N+nab,n_out);

switch continue_tag
    case 0
        
        simGD(1:nab,:)=zeros(nab,2*n_in);
        simGD(nab+1,:)=u_initial;
        
        interpolationGF=[];
        for i=1:nab+1
            interpolationGF(i,:)=y_initial;
        end
        
        simGF(1:nab,:)=interpolationGF(2:nab+1,:);
        
    case 1

        simGD(1:nab+1,:)=continueDATA.GD(end-(nab):end,:);
        
        interpolationGF(1:nab+1,:)=continueDATA.GF(end-(nab):end,:);
        
        simGF(1:nab,:)=interpolationGF(2:nab+1,:);
        
    otherwise
        error('unexpected input: restart_tag');
end

%--------------------------------------------------------------------
% u & y required for the first iteration

switch continue_tag
    case 0
        u=simGD(1+1:1+nab    ,1:n_in);
        y=simGF(1+0:1+nab-1  ,1:n_out)-y_initial;
    
    case 1
        u=simGD(1+1:1+nab    ,1:n_in);
        y=simGF(1+0:1+nab-1  ,1:n_out)-continueDATA.initial;
    
    otherwise
        error('unexpected input: restart_tag');
end


%% Runge Kuta

fprintf('\t\t\t\t');

counter=0;
divergence_tag=0;

for I=1:N
    
    GD_last=simGD(nab+I,:);
    IN=zeros(1,na*n_out+nb*n_in);
   

    for j=1:na
        IN(1,(j-1)*n_out+1:j*n_out)=y(end-j+1,:);
    end
    for j=1:nb
        IN(1,(na*n_out+1+(j-1)*n_in):(na*n_out+j*n_in))=u(end-j+1,:);
    end
    
    OUT_predict=THETA*IN';
    simGF_now=OUT_predict'+y_initial;

    %----------------------------------------------------------------
    
    if any(simGF_now(:)>1e50) || any(isnan(simGF_now(:)))
        fprintf('\n%s\t%s',datetime(datetime,'Format','HH:mm:ss.SSS'),'WARNING!DIVERGENCE!');
        divergence_tag=1;
        save('___0_WARNING_DIVERGENCE','divergence_tag','-ascii');
        break;
    else
        % Note:nothing to do
    end
    
    %----------------------------------------------------------------
    % interpolation of aerodynamic forces

    if (continue_tag==1)||(I>2)
        % continue calculation or start from the third step, use 4th-order interpolation
        force_now=simGF_now;
        force_nextOne=4.*force_now-6.*interpolationGF(end,:)+4.*interpolationGF(end-1,:)-interpolationGF(end-2,:);
        force_nextHalf=1./16.*(35.*force_now-35.*interpolationGF(end,:)+21.*interpolationGF(end-1,:)-5.*interpolationGF(end-2,:));
    else
        % first two steps of no continue calculation
        force_now=simGF_now;
        force_nextOne=2.*force_now-interpolationGF(end,:);
        force_nextHalf=1./2.*(3.*force_now-interpolationGF(end,:));
    end

    %----------------------------------------------------------------
    
    strucForceInput = get_strucForceInput(force_now,Cpc,n_mode,M,lambda,l,h,beta);
    k1=DynamicEquation_RK(GD_last,R0,strucForceInput,n_mode);
    
    K_n=GD_last+0.5*DT*k1;
    strucForceInput = get_strucForceInput(force_nextHalf,Cpc,n_mode,M,lambda,l,h,beta);
    k2=DynamicEquation_RK(K_n,R0,strucForceInput,n_mode);
    
    K_n=GD_last+0.5*DT*k2;
    strucForceInput = get_strucForceInput(force_nextHalf,Cpc,n_mode,M,lambda,l,h,beta);
    k3=DynamicEquation_RK(K_n,R0,strucForceInput,n_mode);
    
    K_n=GD_last+DT*k3;
    strucForceInput = get_strucForceInput(force_nextOne,Cpc,n_mode,M,lambda,l,h,beta);
    k4=DynamicEquation_RK(K_n,R0,strucForceInput,n_mode);
    
    simGD(I+nab+1,:) = GD_last + DT/6*(k1+2*k2+2*k3+k4);
    
    %----------------------------------------------------------------
    
    % update data
    interpolationGF(end+1,:) = simGF_now;
    interpolationGF = interpolationGF(2:end,:);
    simGF(I+nab,:)  = simGF_now;
    
    u(nab+1,  1:n_in)  = simGD(I+nab+1,1:n_in);
    y(nab+1,  1:n_out) = simGF_now-y_initial;
    u=u(2:nab+1,:);
    y=y(2:nab+1,:);
    
    %----------------------------------------------------------------
    % progress bar
    counter=counter+1;
    if counter/N>=0.1
        fprintf('#');
        counter=0;
    else
        % Note:nothing to do
    end
    
end
fprintf('\n');


%% Form generalized displacement to the displacement of the point

N=size(simGD,1)-nab-1;  % Redefine the data length to exclude the initial data of the nab row and only consider the calculated data

w =zeros(N,size(position,2));
dw=zeros(N,size(position,2));
w_monitor =zeros(N,n_monitor);
dw_monitor=zeros(N,n_monitor);

for I=1:N
    for i=1:n_in
        w(I,:)         =  w(I,:)        + sin(i*pi*(l*position)).*simGD(I+nab,i);        % displacement
        w_monitor(I,:) =  w_monitor(I,:)+ sin(i*pi*(l* monitor)).*simGD(I+nab,i);        % displacement of monitor
        dw(I,:)        = dw(I,:)        + sin(i*pi*(l*position)).*simGD(I+nab,i+n_in);
        dw_monitor(I,:)= dw_monitor(I,:)+ sin(i*pi*(l* monitor)).*simGD(I+nab,i+n_in);
    end
end


%% Save data


Gdisp=[...              % generalized displacement
    time,...
    simGD(nab+1:end-1,:)...
    ];
GForce=[...             % generalized aerodynamic force
    time,...
    simGF(nab+1:end,:)...
    ];
disp_monitor=[...       % displacement of monitor
    time,...
    w_monitor,...
    dw_monitor...
    ];


save('keyDATA_predict.mat',...
    'lambda',...
    'monitor',...
    'n_monitor',...
    'position',...
    'DT',...
    'simGD',...
    'simGF'...
    );
save('Gdisp.plt','Gdisp','-ascii');
save('GForce.plt','GForce','-ascii');
save('disp_monitor.plt','disp_monitor','-ascii');


%----------------------------------------------------------------

if divergence_tag~=1
    continueDATA_step=500;    % Note: it should be smaller than the total number of iteration steps
    
    continueDATA.GF            =  simGF(end-continueDATA_step+1:end,:);
    continueDATA.GD            =  simGD(end-continueDATA_step+1:end,:);
    continueDATA.initial       =  y_initial;
    continueDATA.lambda_old     =  lambda;
    continueDATA.Cpc_old         =  Cpc;
    continueDATA.DT_old         =  DT;
    continueDATA.time          =  time(end);
 

    save('___resource_continueDATA/continueDATA.mat','continueDATA');
    
    fprintf('\t\t\t\tcontinueDATA has been saved.\n');
else
    fprintf('\t\t\t\tDivergent continueDATA is not saved.\n');
end


%% Move files

folder_name=[folder_name,datestr(datetime,'_yymmdd_HHMMSS')];

mkdir(folder_name);

movefile(initialFileName,folder_name);

try
    
    movefile('Gdisp.plt',folder_name);
    movefile('GForce.plt',folder_name);
    movefile('disp_monitor.plt',folder_name);
    movefile('keyDATA_predict.mat',folder_name);
    
    if divergence_tag~=1
        copyfile('___resource_continueDATA/continueDATA.mat',folder_name);
    else
        % Note:nothing to do
    end
    
    switch divergence_tag
        case 1
            movefile('___0_WARNING_DIVERGENCE',folder_name);
        otherwise
            % Note:nothing to do
    end
    

catch
    fprintf('%s\tsomething wrong when moving files.\n',datetime(datetime,'Format','HH:mm:ss.SSS'));
end
fprintf('----------------------------------------------\n')
fprintf('%s\tDone\n',datetime(datetime,'Format','HH:mm:ss.SSS'));
fprintf('\t\t\t\t   = %g\n',lambda);
fprintf('\t\t\t\tCpc = %g\n',Cpc);
fprintf('==============================================\n\n')


function strucForceInput = get_strucForceInput(GForce,Cpc,n_mode,M,lambda,l,h,beta)

nm = 1:n_mode;

strucForceInput = ...
    lambda*(l/h)*beta .*( ...
    GForce ...
    -2/(1.4*M^2) .* ((1-cos(nm*pi))./(nm*pi)) * (Cpc-1));

end
